@@ -23,9 +23,20 @@
 #ifndef __LM_INTERNALS_H__
 #define __LM_INTERNALS_H__
 
+#include <glib.h>
+
+#include <sys/types.h>
+
 #include "lm-message.h"
 #include "lm-message-handler.h"
 #include "lm-message-node.h"
+#include "lm-sock.h"
+
+#ifndef G_OS_WIN32
+typedef int LmSocket;
+#else  /* G_OS_WIN32 */
+typedef SOCKET LmSocket;
+#endif /* G_OS_WIN32 */
 
 typedef struct {
 	gpointer       func;
@@ -39,14 +50,14 @@ typedef struct {
 	/* struct to save resolved address */
 	struct addrinfo *resolved_addrs;
 	struct addrinfo *current_addr;
-	int              fd;
-	GIOChannel           *io_channel;
+	LmSocket         fd;
+	GIOChannel      *io_channel;
 } LmConnectData;
 
 void             _lm_connection_failed_with_error (LmConnectData *connect_data,
                                                    int error);
 void             _lm_connection_failed            (LmConnectData *connect_data);
-gboolean         _lm_connection_succeeded (LmConnectData *connect_data);
+void             _lm_connection_succeeded         (LmConnectData *connect_data);
 LmCallback *     _lm_utils_new_callback             (gpointer          func, 
 						     gpointer          data,
 						     GDestroyNotify    notify);
@@ -54,6 +65,7 @@ void             _lm_utils_free_callback            (LmCallback       *cb);
 
 gchar *          _lm_utils_generate_id              (void);
 gchar *          _lm_utils_base64_encode            (const gchar      *str);
+gchar *          _lm_utils_hostname_to_punycode     (const gchar      *hostname);
 const gchar *    _lm_message_type_to_string         (LmMessageType     type);
 const gchar *    _lm_message_sub_type_to_string     (LmMessageSubType  type);
 LmMessage *      _lm_message_new_from_node          (LmMessageNode    *node);
@@ -66,24 +78,29 @@ void             _lm_debug_init                     (void);
 gboolean         _lm_proxy_connect_cb               (GIOChannel *source,
                                                      GIOCondition condition,
                                                      gpointer data);
-void             _lm_ssl_initialize                 (LmSSL            *ssl);
-gboolean         _lm_ssl_begin                      (LmSSL            *ssl,
-						     gint              fd,
-						     const gchar      *server,
-						     GError          **error);
-GIOStatus        _lm_ssl_read                       (LmSSL            *ssl,
-						     gchar            *buf,
-						     gint              len,
-						     gsize             *bytes_read);
-gint             _lm_ssl_send                       (LmSSL            *ssl,
-						     const gchar      *str,
-						     gint              len);
-void             _lm_ssl_close                      (LmSSL            *ssl);
-
 LmHandlerResult    
 _lm_message_handler_handle_message                (LmMessageHandler *handler,
 						   LmConnection     *connection,
 						   LmMessage        *messag);
-
+gboolean         _lm_sock_library_init            (void);
+void             _lm_sock_library_shutdown        (void);
+void             _lm_sock_set_blocking            (LmSocket               sock,
+						   gboolean               block);
+void             _lm_sock_shutdown                (LmSocket               sock);
+void             _lm_sock_close                   (LmSocket               sock);
+LmSocket         _lm_sock_makesocket              (int                    af,
+						   int                    type,
+						   int                    protocol);
+int              _lm_sock_connect                 (LmSocket               sock,
+						   const struct sockaddr *name,
+						   int                    namelen);
+gboolean         _lm_sock_is_blocking_error       (int                    err);
+gboolean         _lm_sock_is_blocking_success     (int                    err);
+int              _lm_sock_get_last_error          (void);
+void             _lm_sock_get_error               (LmSocket               sock, 
+						   void                  *error, 
+						   socklen_t             *len);
+const gchar *    _lm_sock_get_error_str           (int                    err);
+const gchar *    _lm_sock_addrinfo_get_error_str  (int                    err);
 
 #endif /* __LM_INTERNALS_H__ */
