@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2003 Imendio AB 
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -21,8 +21,21 @@
 #include <config.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <time.h>
 
+#include <glib.h>
+
+#ifndef G_OS_WIN32
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_IDN
+#include <stringprep.h>
+#include <punycode.h>
+#include <idna.h>
+#endif
+
 #include "lm-internals.h"
 #include "lm-utils.h"
 
@@ -57,9 +70,14 @@ _lm_utils_free_callback (LmCallback *cb)
 gchar *
 _lm_utils_generate_id (void)
 {
-	static gint  number = 0;
+	static guint  last_id = 0;
+	GTimeVal      tv;
+	glong         val;
 
-	return g_strdup_printf ("msg_%d", ++number);
+	g_get_current_time (&tv);
+	val = (tv.tv_sec & tv.tv_usec) + last_id++;
+		
+	return g_strdup_printf ("%ld%ld", val, tv.tv_usec);
 }
 
 gchar * 
@@ -125,6 +143,44 @@ _lm_utils_base64_encode (const gchar *s)
 	return str;
 }
 
+gchar*
+_lm_utils_hostname_to_punycode (const gchar *hostname)
+{
+#ifdef HAVE_IDN
+	char *s;
+	uint32_t *q;
+	int rc;
+	gchar *result;
+
+	q = stringprep_utf8_to_ucs4 (hostname, -1, NULL);
+	if (q == NULL) {
+		return g_strdup (hostname);
+	}
+
+	rc = idna_to_ascii_4z (q, &s, IDNA_ALLOW_UNASSIGNED);
+	free(q);
+	if (rc != IDNA_SUCCESS) {
+		return g_strdup (hostname);
+	}
+
+	/* insures result is allocated through glib */
+	result = g_strdup(s);
+	free(s);
+
+	return result;
+#else
+	return g_strdup(hostname);
+#endif
+}
+
+/**
+ * lm_utils_get_localtime
+ * @stamp: An XMPP timestamp
+ *
+ * Converts an XMPP timestamp to a struct tm showing local time.
+ * 
+ * Return value: The local time struct.
+ **/
 struct tm *
 lm_utils_get_localtime (const gchar *stamp)
 {
@@ -132,6 +188,8 @@ lm_utils_get_localtime (const gchar *stamp)
 	time_t    t;
 	gint      year, month;
 	
+	g_return_val_if_fail (stamp != NULL, NULL);
+
 	/* 20021209T23:51:30 */
 
 	sscanf (stamp, "%4d%2d%2dT%2d:%2d:%2d", 
@@ -156,3 +214,4 @@ lm_utils_get_localtime (const gchar *stamp)
 	return localtime (&t);
 }
 
+
